import json
import matplotlib.patches as mpatches
import matplotlib.pyplot as plt
import pandas as pd
from matplotlib.pyplot import *
from sympy import *
from sympy.abc import p, v
from tc_python import *

"""
In this example a calibrated double ellipsoidal heat source is used to perform batch steady-state AM calculations
for all the experimental variations of power and scan speed in the single track experiments. 

A printability map is then produced using simple functions for the defects keyholing, balling and lack of fusion.
All defect functions are based on the melt pool dimensions (width, depth and length).
"""


def get_symbolic_function(data):
    expr = None
    if data['interpolationType'] == "CONSTANT":
        expr = data['coefficients'][0] + 1e-30*(p/v) # add 1e-30*p/v in order to match the length of p/v
    elif data["interpolationType"] == "LINEAR":
        expr = (data["coefficients"][0] + data["coefficients"][1] * (p / v))
    elif data["interpolationType"] == "CUBIC":
        expr = (data["coefficients"][0] + data["coefficients"][1] * (p / v) + data["coefficients"][2] * (p / v) ** 2 +
                data["coefficients"][3] * (p / v) ** 3)
    elif data["interpolationType"] == "TWOD_CUBIC":
        expr = (data["coefficients"][0] +
                data["coefficients"][1] * p +
                data["coefficients"][2] * v +
                data["coefficients"][3] * p ** 2 +
                data["coefficients"][4] * v ** 2 +
                data["coefficients"][5] * p * v
                )
    elif data['interpolationType'] == 'SIGMOID':
        expr = (data['coefficients'][0] +
               (data['coefficients'][1] - data['coefficients'][0]) /
               (1 + exp(4 * data['coefficients'][3] * (data['coefficients'][2] - (p / v)) /
               (data['coefficients'][1] - data['coefficients'][0]))))

    expr_below = None
    if data['extrapolationTypeBelow'] == "CONSTANT":
        expr_below = data['coefficientsExtrapolateBelow'][0]
    elif data['extrapolationTypeBelow'] == "LINEAR":
        expr_below = (data['coefficientsExtrapolateBelow'][0] + data['coefficientsExtrapolateBelow'][1] * (p / v))
    elif data['extrapolationTypeBelow'] == "FUNCTION":
        expr_below = expr

    expr_above = None
    if data["extrapolationTypeAbove"] == "CONSTANT":
        expr_above = data["coefficientsExtrapolateAbove"][0]
    elif data["extrapolationTypeAbove"] == "LINEAR":
        expr_above = (data["coefficientsExtrapolateAbove"][0] + data["coefficientsExtrapolateAbove"][1] * (p / v))
    elif data["extrapolationTypeAbove"] == "FUNCTION":
        expr_above = expr

    args = []
    args.append((expr_below, (p / v) < data["minValue"]))
    args.append((expr_above, (p / v) > data["maxValue"]))
    args.append((expr, True))
    symb_function = Piecewise(*args)

    f = lambdify([p, v], symb_function, "numpy")
    return f


class HeatSourceType(Enum):
    """
    The heat soure type
    """
    DOUBLE_ELLIPSOIDAL = 0
    """
    Ellipsoidal
    """
    CONICAL = 1
    """
    Conical
    """
    GAUSSIAN = 2
    """
    Gaussian
    """


# The calibrated heat sources are currently not available directly on TC-Python 2024a
# As a workaround we therefore create the functions directly in python using the
# heat sources configuration file. This is not recommended since the format
# for the configuration files will probably change between releases. Objects for the
# calibrated heat sources will be available in upcoming releases of the AM module for
# TC-Python.
#
# R1 (ar, re)
# R2 (af, ri)
# R3 (b)
# R4 (c,h)
# Linear: coeff0 + coeff1*p/v
# Cubic: coeff0 + coeff1*p/v +coeff2*(p/v)2 +coeff3*(p/v)3
# Sigmoid: coeff0 + (coeff1 - coeff0)/(1 + Math.exp(4*coeff3*(coeff2 - x)/(coeff1 - coeff0 )))
# 2dCubic: coeff0 + P*coeff1+ v*coeff2 + P2*coeff3+v2*coeff4+P*v*coeff5


class CalibratedHeatSourceFunctions(object):
    """
    Calibrated heat source
    """

    def __init__(self, parameter_file):
        self.fun = {}
        self.power = 100.0
        self.scan_speed = 0.5

        f = open(parameter_file)
        hs = json.load(f)

        if hs["heatSourceType"] == "DOUBLE_ELLIPSOIDAL":
            self.type = HeatSourceType.DOUBLE_ELLIPSOIDAL
            self.fun["absorptivity"] = get_symbolic_function(hs["absorptivityData"])
            self.fun["ar"] = get_symbolic_function(hs["r1Data"])
            self.fun["af"] = get_symbolic_function(hs["r2Data"])
            self.fun["b"] = get_symbolic_function(hs["r3Data"])
            self.fun["c"] = get_symbolic_function(hs["r4Data"])

        elif hs["heatSourceType"] == "CONICAL":
            self.type = HeatSourceType.CONICAL
            self.fun["absorptivity"] = get_symbolic_function(hs["absorptivityData"])
            self.fun["re"] = get_symbolic_function(hs["r1Data"])
            self.fun["ri"] = get_symbolic_function(hs["r2Data"])
            self.fun["b"] = get_symbolic_function(hs["r3Data"])
            self.fun["h"] = get_symbolic_function(hs["r4Data"])
        elif hs["heatSourceType"] == "GAUSSIAN":
            self.type = HeatSourceType.GAUSSIAN
            self.fun["absorptivity"] = get_symbolic_function(hs["absorptivityData"])
            self.fun["r"] = get_symbolic_function(hs["r1Data"])

    def set_scan_speed(self, scan_speed):
        self.scan_speed = scan_speed

    def set_power(self, power):
        self.power = power

    def get_type(self) -> HeatSourceType:
        return self.type

    def get_absorptivity(self):
        return self.fun["absorptivity"](self.power, self.scan_speed)

    def get_c(self):
        return self.fun["c"](self.power, self.scan_speed) * 1e-6

    def get_ar(self):
        return self.fun["ar"](self.power, self.scan_speed) * 1e-6

    def get_af(self):
        return self.fun["af"](self.power, self.scan_speed) * 1e-6

    def get_b(self):
        return self.fun["b"](self.power, self.scan_speed) * 1e-6

    def get_re(self):
        return self.fun["re"](self.power, self.scan_speed) * 1e-6

    def get_ri(self):
        return self.fun["ri"](self.power, self.scan_speed) * 1e-6

    def get_h(self):
        return self.fun["h"](self.power, self.scan_speed) * 1e-6

    def get_r(self):
        return self.fun["r"](self.power, self.scan_speed) * 1e-6


def get_depth_at_hatch_distance(result: SteadyStateResult, hatch_distance: float, liquidus: float):
    """
    Iso contour of liquidus at hatch distance to get penetration depth
    """
    mesh = result.get_pyvista_mesh()
    origin = mesh.center
    origin[1] = hatch_distance * 1e-6
    slice = mesh.slice(normal="y", origin=origin)
    contour = slice.contour(isosurfaces=np.array([liquidus]))
    if contour.n_points > 0:
        depth_at_hatch_distance = contour.bounds[4]
    else:
        depth_at_hatch_distance = 0.0
    return abs(depth_at_hatch_distance)


def plot_heat_source_parameters(df, heat_source_functions: CalibratedHeatSourceFunctions):
    fig_hs, axs_hs = plt.subplots(1, 1)

    scan_speed = df["speed (mm/s)"]
    power = df["power (W)"]
    heat_source_functions.set_power(power)
    heat_source_functions.set_scan_speed(scan_speed)

    axs_hs.plot((power / scan_speed), heat_source_functions.get_absorptivity(), label="abs", marker="o", lw=0)
    axs_hs.plot((power / scan_speed), heat_source_functions.get_ar() * 1e6, label="ar", marker="o", lw=0)
    axs_hs.plot((power / scan_speed), heat_source_functions.get_af() * 1e6, label="af", marker="o", lw=0)
    axs_hs.plot((power / scan_speed), heat_source_functions.get_b() * 1e6, label="b", marker="o", lw=0)
    axs_hs.plot((power / scan_speed), heat_source_functions.get_c() * 1e6, label="c", marker="o", lw=0)
    axs_hs.set_xlabel("Energy density (J/mm)")
    axs_hs.set_ylabel("Size (\u03bcm) / absorptivity")
    axs_hs.set_ylim(bottom=0.0)
    axs_hs.legend()


def plot_exp_and_calculated_width_depth(df):
    fig, axs = plt.subplots(2, 1)
    axs[0].errorbar(df['energy density (J/mm)'], df['meltpool depth (um)'], yerr=df['depth error (um)'], marker="o", linestyle="None", capsize=7.0, label='Experiment', zorder=0)
    axs[0].plot(df["energy density (J/mm)"], df["calc depth (um)"], label="Calculation", marker="<", lw=0)
    axs[0].set_xlabel("Energy density (J/mm)")
    axs[0].set_ylabel("Depth (\u03bcm)")
    axs[0].set_ylim(bottom=0.0)
    axs[0].legend()

    if "meltpool width (um)" in df:
        axs[1].errorbar(df['energy density (J/mm)'], df['meltpool width (um)'], yerr=df['width error (um)'], marker="o", linestyle="None", capsize=7.0, label='Experiment', zorder=0)
    axs[1].plot(df["energy density (J/mm)"], df["calc width (um)"], label="Calculation", marker="<", lw=0)
    axs[1].set_xlabel("Energy density (J/mm)")
    axs[1].set_ylabel("Width (\u03bcm)")
    axs[1].set_ylim(bottom=0.0)
    axs[1].legend()


def plot_calculated_melt_pool_dimensions(df):
    fig, axs2 = plt.subplots(3, 1)

    cntr1 = axs2[0].tricontourf(df["speed (mm/s)"], df["power (W)"], df["calc depth (um)"], levels=14, cmap="RdBu_r")
    fig.colorbar(cntr1, ax=axs2[0])
    axs2[0].plot(df["speed (mm/s)"], df["power (W)"], "ko", ms=3)
    axs2[0].set_title("Calculated melt pool depth (%d points)" % len(df.index))
    axs2[0].set_xlabel("Scan speed (mm/)")
    axs2[0].set_ylabel("Power (W)")

    cntr2 = axs2[1].tricontourf(df["speed (mm/s)"], df["power (W)"], df["calc width (um)"], levels=14, cmap="RdBu_r")
    fig.colorbar(cntr2, ax=axs2[1])
    axs2[1].plot(df["speed (mm/s)"], df["power (W)"], "ko", ms=3)
    axs2[1].set_title("Calculated melt pool width (%d points)" % len(df.index))
    axs2[1].set_xlabel("Scan speed (mm/)")
    axs2[1].set_ylabel("Power (W)")

    cntr2 = axs2[2].tricontourf(df["speed (mm/s)"], df["power (W)"], df["calc length (um)"], levels=14, cmap="RdBu_r")
    fig.colorbar(cntr2, ax=axs2[2])
    axs2[2].plot(df["speed (mm/s)"], df["power (W)"], "ko", ms=3)
    axs2[2].set_title("Calculated melt pool length (%d points)" % len(df.index))
    axs2[2].set_xlabel("Scan speed (mm/)")
    axs2[2].set_ylabel("Power (W)")
    plt.subplots_adjust(hspace=0.7)


def plot_defects(df):
    fig, axs2 = plt.subplots(2, 2)

    cntr2 = axs2[0, 0].tricontourf(df["speed (mm/s)"], df["power (W)"], df["keyholing"], levels=14, cmap="RdBu_r")
    fig.colorbar(cntr2, ax=axs2[0, 0])
    axs2[0, 0].set_title("Keyholing (W/D)")
    axs2[0, 0].set_xlabel("Scan speed (mm/)")
    axs2[0, 0].set_ylabel("Power (W)")

    cntr2 = axs2[0, 1].tricontourf(df["speed (mm/s)"], df["power (W)"], df["balling"], levels=14, cmap="RdBu_r")
    fig.colorbar(cntr2, ax=axs2[0, 1])
    axs2[0, 1].set_title("Balling (W/L)")
    axs2[0, 1].set_xlabel("Scan speed (mm/)")
    axs2[0, 1].set_ylabel("Power (W)")

    cntr1 = axs2[1, 0].tricontourf(df["speed (mm/s)"], df["power (W)"], df["lack of fusion"], levels=14, cmap="RdBu_r")
    fig.colorbar(cntr1, ax=axs2[1, 0])
    axs2[1, 0].set_title("Lack of fusion (D/t)")
    axs2[1, 0].set_xlabel("Scan speed (mm/)")
    axs2[1, 0].set_ylabel("Power (W)")

    cntr2 = axs2[1, 1].tricontourf(df["speed (mm/s)"], df["power (W)"], df["lack of fusion - hatch"], levels=14,
                                   cmap="RdBu_r")
    fig.colorbar(cntr2, ax=axs2[1, 1])
    axs2[1, 1].set_title("lack of fusion -  (D_hatch/t)")
    axs2[1, 1].set_xlabel("Scan speed (mm/)")
    axs2[1, 1].set_ylabel("Power (W)")

    plt.subplots_adjust(hspace=0.5, wspace=0.4)


def plot_printability_map(df, df_experiments_only):
    fig, ax = plt.subplots(1, 1)
    ax.title.set_text("Layer thickness:{}\u03bcm, hatch distance:{}\u03bcm".format(powder_thickness,
                                                                                   hatch_distance))

    alpha = 0.5
    levels_keyholing = [0.0, keyhole_limit]
    levels_balling = [0.0, balling_limit]
    levels_lack_of_fusion = [0.0, lack_of_fusion_limit]
    levels_lack_of_fusion_hatch = [0.0, lack_of_fusion_hatch_limit]

    ax.tricontourf(df["speed (mm/s)"], df["power (W)"], df["balling"], levels=levels_balling, colors=("magenta",),
                   alpha=alpha)
    ax.tricontourf(df["speed (mm/s)"], df["power (W)"], df["lack of fusion"], levels=levels_lack_of_fusion,
                   colors=("blue",), alpha=alpha)
    ax.tricontourf(df["speed (mm/s)"], df["power (W)"], df["keyholing"], levels=levels_keyholing, colors=("teal",),
                   alpha=alpha)
    ax.tricontourf(df["speed (mm/s)"], df["power (W)"], df["lack of fusion - hatch"],
                   levels=levels_lack_of_fusion_hatch, colors=("deepskyblue",), alpha=alpha)

    c = ax.tricontour(df["speed (mm/s)"], df["power (W)"], df["keyholing"],
                      levels=np.linspace(.0, keyhole_limit, 8))
    ax.clabel(c, inline=True, fontsize=10)

    for i, row in df_experiments_only.iterrows():
        ax.text(x=row["speed (mm/s)"], y=row["power (W)"], s=row["text"], color=row["color"], ha="center", va="center")

    ax.set_xlabel("Scan speed (mm/s)")
    ax.set_ylabel("Power (W)")
    lack_of_fusion_hatch_patch = mpatches.Patch(color="deepskyblue",
                                                label="Lack of fusion - hatch (D_hatch/t < {})".format(
                                                    lack_of_fusion_hatch_limit), alpha=alpha)
    lack_of_fusion_patch = mpatches.Patch(color="blue", label="Lack of fusion (D/t < {})".format(lack_of_fusion_limit),
                                          alpha=alpha)
    balling_patch = mpatches.Patch(color="magenta", label="Balling (W/L < {})".format(balling_limit), alpha=alpha)
    keyholing_patch = mpatches.Patch(color="teal", label="Keyholing (W/D < {})".format(keyhole_limit), alpha=alpha)
    ax.legend(handles=[keyholing_patch, balling_patch, lack_of_fusion_patch, lack_of_fusion_hatch_patch])


with TCPython(logging_policy=LoggingPolicy.SCREEN) as start:
    start.set_cache_folder(os.path.basename(__file__) + "cache")

    # experimental data for SS316L from Jun Wei et al. Advances in Materials Science and Engineering Volume 2019
    # https://doi.org/10.1155/2019/9451406
    exp_file = "316L_single_track_experiments.csv"
    powder_thickness = 10.0  # in micro-meter
    hatch_distance = 5.0  # in micro-meter. Only single track experiment but let's use a hatch spacing for the
    # printability map
    ambient_temperature = 353  # in K, as given in the paper

    heat_source_file = HeatSource.get_path_of_calibrated_heatsource('Double ellipsoidal - 316L - beam d 15um')
    heat_source_functions = CalibratedHeatSourceFunctions(heat_source_file)
    heat_source_type = heat_source_functions.get_type()

    df = pd.read_csv(exp_file, skipinitialspace=True)
    print(df.to_string())

    extend_dataset_all_combinations = True
    # extend the dataset for calculation of the printability map by adding all combinations of the laser power
    # and scan speed
    if extend_dataset_all_combinations:
        power_list = df["power (W)"]
        u_list = df["speed (mm/s)"]
        XX, YY = np.meshgrid(power_list, u_list, indexing="ij")
        NT = np.prod(XX.shape)
        data = {"power (W)": np.reshape(XX, NT),
                "speed (mm/s)": np.reshape(YY, NT)}
        df2 = pd.DataFrame(data=data)
        df = pd.concat([df, df2])
        df = df.drop_duplicates(subset=["power (W)", "speed (mm/s)"], keep="first", ignore_index=True)
        print(df.to_string())
    else:
        df = df

    if (heat_source_type == HeatSourceType.DOUBLE_ELLIPSOIDAL):
        heat_source = HeatSource.double_ellipsoidal()
    elif (heat_source_type == HeatSourceType.CONICAL):
        heat_source = HeatSource.double_ellipsoidal()
    else:
        heat_source = HeatSource.gaussian().with_keyhole_model(KeyholeModel())

    mp = MaterialProperties.from_library("SS316L")

    am_calculator = (start.with_additive_manufacturing()
                     .with_steady_state_calculation()
                     .with_numerical_options(NumericalOptions().set_number_of_cores(4))
                     .disable_fluid_flow_marangoni()
                     .with_material_properties(mp)
                     .with_heat_source(heat_source)
                     .with_mesh(Mesh().coarse()))
    am_calculator.set_ambient_temperature(ambient_temperature)
    am_calculator.set_base_plate_temperature(ambient_temperature)

    # add the energy density and dataframe columns for calculated melt pool dimensions
    df["energy density (J/mm)"] = df["power (W)"] / df["speed (mm/s)"]
    df["calc width (um)"] = np.nan
    df["calc depth (um)"] = np.nan
    df["calc length (um)"] = np.nan
    df["hatch distance depth (um)"] = np.nan

    for i, row in df.iterrows():
        power = row["power (W)"]
        scan_speed = row["speed (mm/s)"]
        print("Row:{}/{}, Power:{}W, scan speed:{}mm/s".format(i, len(df.index), power, scan_speed))

        heat_source.set_power(power)
        heat_source.set_scanning_speed(scan_speed * 1e-3)

        heat_source_functions.set_power(power)
        heat_source_functions.set_scan_speed(scan_speed)

        heat_source.set_absorptivity(heat_source_functions.get_absorptivity())
        if (heat_source_type == HeatSourceType.DOUBLE_ELLIPSOIDAL):
            heat_source.set_ar(heat_source_functions.get_ar())
            heat_source.set_af(heat_source_functions.get_af())
            heat_source.set_b(heat_source_functions.get_b())
            heat_source.set_c(heat_source_functions.get_c())
        elif (heat_source_type == HeatSourceType.CONICAL):
            heat_source.set_re(heat_source_functions.get_re())
            heat_source.set_ri(heat_source_functions.get_ri())
            heat_source.set_h(heat_source_functions.get_h())
        else:
            heat_source.set_beam_radius(heat_source_functions.get_r())

        try:
            result = am_calculator.calculate()
            length = result.get_meltpool_length()
            width = result.get_meltpool_width()
            depth = result.get_meltpool_depth()

            if length != float("inf") and width != float("inf") and depth != float("inf"):
                df.loc[i, ["hatch distance depth (um)"]] = (get_depth_at_hatch_distance(result, hatch_distance,
                                                                                        mp.get_liquidus_temperature())
                                                            * 1e6)
                df.loc[i, ["calc width (um)"]] = width * 1e6
                df.loc[i, ["calc depth (um)"]] = depth * 1e6
                df.loc[i, ["calc length (um)"]] = length * 1e6
        except Exception as e:
            print(e)

    # drop the rows with failed calculations, if any
    df.dropna(subset=["calc width (um)", "calc depth (um)", "calc length (um)"], inplace=True)

    # filter the rows that include experimental data
    df_experiments_only = df.dropna(subset=["meltpool depth (um)", "meltpool width (um)"], inplace=False)

    # the publication below reference several limits for the defects based on melt pool dimensions:
    # Sheikh et al. "High-throughput Alloy and Process Design for Metal Additive Manufacturing"
    # Condensed Matter Jan 13, 2023, https://arxiv.org/abs/2304.04149
    lack_of_fusion_limit = 1.5  # eq. 15 when melt pool depth / powder thickness < 1.0
    lack_of_fusion_hatch_limit = 1.1  # melt pool depth at hatch distance < powder thickness
    balling_limit = 0.23  # balling: eq. 18 when w/L<0.43 or eq. 19 when L/w<0.26
    keyhole_limit = 1.9  # keyholing: eq. 20 when melt pool width / depth < 2.5. Generally a transition at 1.0
    # and keyhole at 1.5)

    df["lack of fusion - hatch"] = df["hatch distance depth (um)"] / powder_thickness
    df["lack of fusion"] = df["calc depth (um)"] / powder_thickness
    df["balling"] = df["calc width (um)"] / df["calc length (um)"]
    df["keyholing"] = df["calc width (um)"] / df["calc depth (um)"]

    plot_exp_and_calculated_width_depth(df_experiments_only)
    plot_heat_source_parameters(df, heat_source_functions)
    plot_calculated_melt_pool_dimensions(df)
    plot_defects(df)
    plot_printability_map(df, df_experiments_only)

    print(df.to_string())

    plt.show()
